package config

import (
	"flag"

	"github.com/BurntSushi/toml"
)

type Config struct {
	Debug  bool         `toml:"debug"`
	PGSQL  PGSQLConfig  `toml:"pgsql"`
	Server ServerConfig `toml:"server"`
	TGBot  TGBotConfig  `toml:"tg_bot"`
}

type PGSQLConfig struct {
	Host        string `toml:"host"`
	Port        int    `toml:"port"`
	User        string `toml:"user"`
	Password    string `toml:"password"`
	Database    string `toml:"database"`
	MaxConn     int    `toml:"max_conn"`
	MaxIdleConn int    `toml:"max_idle_conn"`
	EnableLog   bool   `toml:"enable_log"`
	CertFile    string `toml:"cert_file"`
}

type ServerConfig struct {
	Listen string `toml:"listen"`
}

type TGBotConfig struct {
	Token string `toml:"token"`
}

var confPath = flag.String("c", "config.toml", "config file path")

func New() (config *Config, err error) {
	config = new(Config)
	_, err = toml.DecodeFile(*confPath, config)
	return
}
