package dao

import (
	"fmt"
	"os"
	"sdk_api/config"
	dbModel "sdk_api/model/db"
	"time"

	"gorm.io/driver/postgres"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
	"gorm.io/gorm/schema"
)

type Dao struct {
	c  *config.Config
	db *gorm.DB
}

func New(_c *config.Config) (dao *Dao, err error) {
	dao = &Dao{
		c: _c,
	}

	// dsn := fmt.Sprintf("%s:%s@tcp(%s:%d)/%s?charset=utf8mb4&parseTime=True",
	// 	_c.MySQL.User, _c.MySQL.Password, _c.MySQL.Host, _c.MySQL.Port, _c.MySQL.Database)

	// dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%d sslmode=verify-ca sslrootcert=ca.crt",
	dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%d",
		_c.PGSQL.Host, _c.PGSQL.User, _c.PGSQL.Password, _c.PGSQL.Database, _c.PGSQL.Port,
	)

	if _c.PGSQL.CertFile != "" {
		dsn = fmt.Sprintf("%s sslmode=require sslrootcert=%s", dsn, _c.PGSQL.CertFile)
	}

	lgr := logger.Default
	if _c.PGSQL.EnableLog {
		lgr = logger.Default.LogMode(logger.Info)
	}

	dao.db, err = gorm.Open(postgres.Open(dsn), &gorm.Config{
		NamingStrategy: schema.NamingStrategy{
			SingularTable: true,
		},
		Logger: lgr,
	})
	if err != nil {
		return
	}
	sqlDB, err := dao.db.DB()
	if err != nil {
		return
	}
	sqlDB.SetMaxOpenConns(_c.PGSQL.MaxConn)
	sqlDB.SetMaxIdleConns(_c.PGSQL.MaxIdleConn)
	sqlDB.SetConnMaxIdleTime(time.Hour)

	migrate := os.Getenv("MIGRATE")
	if migrate == "true" {
		err = dao.db.AutoMigrate(&dbModel.User{}, &dbModel.Active{}, &dbModel.ChatGroup{})
		if err != nil {
			return
		}
	}

	err = dao.addTestGroup()
	if err != nil {
		return
	}

	return dao, nil
}
